#!/usr/local/bin/perl -w

# Filter.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directories (input, output)

$InputDirectory = 'Transcribed';
$OutputDirectory = 'Filtered';

# Delimiters bracketing text to be discarded

@Delimiters = (
	'{', '}',
	'|', '|'
	);

# Substitutions

@Substitutions = (
	'[/fn]', 'N',
	'[di]I[/di]', 'J',
	'[di]U[/di]', 'V',
	'[di]UI[/di]', 'VJ',
	);

# Items to be discarded

@Items = (
	'[st]', '[/st]', '[it]', '[/it]', '[kc]', '[/kc]',
	'[sc]', '[/sc]', '[ns]', '[/ns]', '[fn]', '[/fn]',
	'[rb]', '[/rb]', '[sb]', '[/sb]', '[di]', '[/di]'
	);

# END SPECIFICATIONS

# Print introduction

print "Filter: Filter unwanted items from transcriptions.\n\n";

print "Note:\n\n";

print "(1) Newline conversion: ASCII 13 is changed to ASCII 10\n";
print "or vice versa, depending on the operating system.\n\n";

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input directory: $InputDirectory.\n";
print "Output directory: $OutputDirectory.\n\n";

for($i = 0; $i < $#Delimiters; $i += 2)
{ print $Delimiters[$i], "This will be discarded",
	$Delimiters[$i + 1], "\n" }
print "\n";

for($i = 0; $i < $#Substitutions; $i += 2)
{ print $Substitutions[$i], " will be replaced with ",
	$Substitutions[$i + 1], "\n" }
print "\n";

print "These items will then be discarded:\n";
$Length = 0;
$i = 0;
while($i <= $#Items)
{
	print $Items[$i];
	print " ";
	$Length += length($Items[$i]) + 2;
	if($Length > 60)
	{
		print "\n";
		$Length = 0;
	}
	$i += 1;
}
print "\n";

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known\n" }

# Make paths to directories

$InputPath = $Dir.$InputDirectory.$Dir;
$OutputPath = $Dir.$OutputDirectory.$Dir;
unless($System eq 'MAC')
{
	$InputPath = '.'.$InputPath;
	$OutputPath = '.'.$OutputPath;
}

# Make output directory

mkdir($OutputDirectory, 0770);

# Open input directory

opendir(INDIR, $InputDirectory) or die
	"Can't open input directory $InputDirectory";

# Make input directory contents list

@Contents = readdir(INDIR);

# Remove . and .. from directory list

unless($System eq 'MAC') { splice(@Contents, 0, 2) }

# FILTER TRANSCRIPTIONS

print "\nFilter transcriptions:\n";

foreach $File (@Contents)
{
	print "$InputPath$File -> $OutputPath$File\n";

	# Read input file

	open(INPUT, '<'.$InputPath.$File) or die
	"Can't read $InputPath$File";

	@List = <INPUT>;

	# Close file

	close(INPUT);

	# Make output string

	$String = join('', @List);

	# Convert newlines

	if($System eq 'UNIX') { $Find = chr(13) }
	elsif($System eq 'MAC') { $Find = chr(10) }

	$String =~ s/$Find/\n/g;

	# Discard delimited text

	for($i = 0; $i <= $#Delimiters; $i += 2)
	{
		$Open = quotemeta($Delimiters[$i]);
		$Close = quotemeta($Delimiters[$i + 1]);
		$String =~ s/$Open[\S\W]*?$Close//g;
	}

	# Perform substitutions

	for($i = 0; $i <= $#Substitutions; $i += 2)
	{
		$Find = quotemeta($Substitutions[$i]);
		$Replace = $Substitutions[$i + 1];
		$String =~ s/$Find/$Replace/g;
	}

	# Discard unwanted items

	foreach $Item (@Items)
	{
		$Find = quotemeta($Item);
		$String =~ s/$Find//g;
	}

	# Remove multiple spaces

	$String =~ tr/ / /s;

	# Write output file

	open(OUTPUT, '>'.$OutputPath.$File) or die
	"Can't write $OutputPath$File";

	# Write output to file

	print OUTPUT $String;

	# Close file

	close(OUTPUT);
}

# Print end message

$Duration = time - $^T;

print "\nFilter finished in $Duration seconds.\n";
