#!/usr/local/bin/perl -w

# Check.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directory (input)

$InputDirectory = 'Transcribed';

# Division marker delimiters

@Division = (
	'<',	# Open delimiter
	'>'		# Close delimiter
	);

# Tag delimiters

@Tag = (
	'[',	# Start-tag open delimiter
	'[/',	# End-tag open delimiter
	']'		# Tag close delimiter
	);

# END SPECIFICATIONS

# Print intoduction

print "Check: Detect tag errors.\n\n";

print "Notes:\n\n";

print "Generic identifiers must be alphanumeric.\n\n";

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input directory: $InputDirectory.\n\n";

# Make division marker components

$DMO = $Division[0];	# Division Marker Open
$DMC = $Division[1];	# Division Marker Close

print "Division markers should look like this:\n",
	$DMO, "DIVISION MARKER", $DMC, "\n\n";

# Make tag components
	
$TSO = $Tag[0];	# Tag Start Open
$TEO = $Tag[1];	# Tag End Open
$TC = $Tag[2];	# Tag Close
	
print "Tags should look like this (gi = generic identifier):\n",
	$TSO, "gi", $TC, "TAGGED TEXT", $TEO, "gi", $TC, "\n\n";

# Make patterns for regular expressions
	
$DMO = quotemeta($DMO);
$DMC = quotemeta($DMC);
$TSO = quotemeta($TSO);
$TEO = quotemeta($TEO);
$TC = quotemeta($TC);

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known\n" }

# Make path to directory

$InputPath = $Dir.$InputDirectory.$Dir;
unless($System eq 'MAC') { $InputPath = '.'.$InputPath }

# Open input directory

opendir(INDIR, $InputDirectory) or die
	"Can't open input directory $InputDirectory";

# Make directory contents list

@Contents = readdir(INDIR);

# Remove . and .. from directory list

unless($System eq 'MAC') { splice(@Contents, 0, 2) }

# CHECK TRANSCRIPTIONS

print "Check transcriptions:\n";

foreach $InputName (@Contents)
{
	# Open input file for reading
	
	open(INPUT, '<'.$InputPath.$InputName) or die
		"Can't open input file $InputPath$InputName\n";

	print "$InputPath$InputName\n";
	
	# Make input into one string
	
	@InputList = <INPUT>;
	$Input = join('', @InputList);
	
	# Close file
	
	close(INPUT);
	
	# Make list of generic identifiers
		
	@Identifiers = &ListIdentifiers($Input);
	
	# Check input
	
	foreach $GI (@Identifiers)
	{
		# Check for block markers within matching tags
		
		while($Input =~ /$TSO$GI$TC([\S\W]*?)$TEO$GI$TC/g)
		{
			$Check = $1;
			if($Check =~ /$DMO.*?$DMC/)
			{ die "Block marker within $GI tags: $Check\n" }
		}
		
		# Remove matching start and end tags
		
		$Input =~ s/$TSO$GI$TC([\S\W]*?)$TEO$GI$TC/$1/g;
	}
	
	# Check for remaining tags
	
	if($Input =~ /([\S\W]{0,20}[$TSO$TEO$TC][\S\W]{0,20})/g)
	{ die "Tag error: $1\n" }
}

# Print end message
	
$Duration = time - $^T;

print "\nCheck finished in $Duration seconds.\n";

sub ListIdentifiers # Make compact list of generic identifiers
{
	# Make list of generic identifiers from start tags
	
	my(@Identifiers) = ();
	
	push(@Identifiers, $_[0] =~ /$TSO(\w*?)$TC/g);
	
	# Sort and compact list
	
	@Identifiers = sort @Identifiers;
	
	my($Identifier) = '';
	my($Previous) = '';
	my(@List) = ();
	
	foreach $Identifier (@Identifiers)
	{
		if($Identifier ne $Previous)
		{ push(@List, $Identifier) }
		
		$Previous = $Identifier;
	}
	
	return @List;
}
