#!/usr/local/bin/perl -w

# DM.pl

# SPECIFICATIONS

# Input file (data matrix)

$Data = 'M81';

# END SPECIFICATIONS

# Print introduction

print "Make dissimilarity matrix from data matrix and labels.\n";
print "Dissimilarities calculated with simple matching coeff.\n";

print "\n$Data\n";

# Open files

$Labels = $Data.'_labels';	# Name labels file

open(DATA, "$Data") or die "Can't open $Data";
open(LABELS, "$Labels") or die "Can't open $Labels";

# Read data matrix file

print "\nRead data matrix\n";

@DataMat = ();
$Rows = 0;

while(<DATA>)
{
	$Vector = $_;
	@Elements = ($Vector =~ /(\d)\s/g);
	push(@DataMat, [ @Elements ]);
	$Rows += 1;
}

$Columns = @Elements;
print "Dimensions: $Rows rows x $Columns columns.\n";

# Transpose data matrix

print "\nTranspose data matrix\n";

@TransMat = ();

for($N = 0; $N < $Rows; $N += 1)
{
	for($P = 0; $P < $Columns; $P += 1)
	{ $TransMat[$P][$N] = $DataMat[$N][$P] }
}

# Construct dissimilarity matrix

print "\nConstruct dissimilarity matrix\n";

@DissMat = ();

for($R = 0; $R < $Columns; $R += 1)
{
	print $R + 1, "\n";
	
	# Get reference data matrix vector
	
	@RV = @{$TransMat[$R]};
	
	# Enter dissimilarities for each column
	
	for($C = 0; $C < $Columns; $C += 1)
	{
		# Get corresponding data matrix vector
		
		@CV = @{$TransMat[$C]};
		
		# Count number of matches
		
		$Matches = 0;
		
		for($N = 0; $N < $Rows; $N += 1)
		{ if($RV[$N] == $CV[$N]) { $Matches += 1 } }
		
		# Calculate simple matching coefficient
		
		$SMC = $Matches/$Rows;
		
		# Enter dissimilarity
		
		$DissMat[$R][$C] = 1 - $SMC;
	}
}

# Read labels file

print "\nRead labels file\n";

$C = 0;
@List = ();

while($C < $Columns)
{
	$Vector = <LABELS>;
	@Elements = ($Vector =~ /(\S{1,8})\s*/g);
	push(@List, @Elements);
	$C += @Elements;
}

print "$C labels.\n";

# Add labels

print "\nAdd labels\n";

#Add row labels

for($R = 0; $R < $Columns; $R += 1)
{ unshift(@{$DissMat[$R]}, $List[$R]) }

#Add column labels

unshift(@List, 'R\C');
unshift(@DissMat, [ @List ]);
shift(@List);

# Write dissimilarity matrix file

$Output = $Data.'_DM';	# Name output file

open(OUTPUT, ">$Output") or die "Can't write to $Output";

for($R = 0; $R <= $Columns; $R += 1)
{ print OUTPUT join("\t", @{$DissMat[$R]}), "\n" }

# Print end message
	
$Duration = time - $^T;

print "\nDM finished in $Duration seconds.\n";
printf "\nPerformance index: %d.\n", $Rows*$Columns/$Duration;
