#!/usr/local/bin/perl -w

# LI.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directories (input, output)

$InputDirectory = 'Formatted';
$OutputDirectory = 'Lists';

# Division marker delimiters

@Division = (
	'<',	# Open delimiter
	'>'		# Close delimiter
	);

# Certainty threshold

$Threshold = 0.051;

# END SPECIFICATIONS

# Print introduction

print "List Items:\n\n";

print "For each input file:\n";
print "- output a list of distinct items\n";
print "- count certain and uncertain items.\n\n";

print "Notes:\n\n";

print "(1) Input files must be properly formatted.\n";
print "    (The Format or Normalise utility does this.)\n";
print "(2) An item is counted as uncertain if its certainty is\n";
print "    below a specified threshold.\n\n";

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known\n" }

# Make paths to directories

$InputPath = $Dir.$InputDirectory.$Dir;
$OutputPath = $Dir.$OutputDirectory.$Dir;
unless($System eq 'MAC')
{
	$InputPath = '.'.$InputPath;
	$OutputPath = '.'.$OutputPath;
}

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input directory: $InputDirectory.\n";
print "Output directory: $OutputDirectory.\n\n";

print "Division markers should look like this:\n",
	"$Division[0]DIVISION MARKER$Division[1]\n\n";

print "Any item with a certainty below $Threshold\n";
print "will be counted as uncertain.\n\n";

# Make patterns for regular expressions

$DMO = quotemeta($Division[0]);
$DMC = quotemeta($Division[1]);

# Open input directory

opendir(INDIR, $InputDirectory) or die
	"Can't open input directory $InputDirectory";

# Make input directory contents list

@Contents = readdir(INDIR);

# Remove . and .. from directory list

unless($System eq 'MAC') { splice(@Contents, 0, 2) }

# Make output directory

mkdir($OutputDirectory, 0770);

# MAKE LISTS FOR EACH INPUT FILE

print "Make lists:\n\n";

foreach $InputName (@Contents)
{
	# Initialise
	
	%Array = ();
	$Certs = 0;
	$Uncerts = 0;
	
	# Open output file for writing
	
	open(OUTPUT, ">$OutputPath$InputName") or die
		"Can't write to $OutputPath$InputName";
	
	# Read input file
	
	open(INPUT, "<$InputPath$InputName") or die
		"Can't open input file $InputPath$InputName";
	
	print "$InputPath$InputName -> $OutputPath$InputName\n";
	
	@List = <INPUT>;
	
	# Close file
	
	close(INPUT);
	
	# Remove newlines from input
	
	chomp(@List);
	
	# List items
	
	foreach $Line (@List)
	{
		# Skip div. markers
		
		if($Line =~ /^$DMO.*$DMC$/) { next }
		
		# Check format
		
		unless($Line =~ /(.+)\t(.+)/)
		{ die "Error: Incorrect input format: $Line" }
		
		# Get item and certainty
		
		$Item = $1;
		$Cert = $2;
		
		# Increment counters
		
		if($Cert < $Threshold) { $Uncerts += 1 }
		else { $Certs += 1 }
		
		# Add item to array if not already there
		
		unless(exists($Array{$Item}))
		{ $Array{$Item} = $Item }
	}
	
	# Print number of items
	
	print "Certain: $Certs  Uncertain: $Uncerts\n";
	
	# Write output file
	
	print OUTPUT join("\n", sort(keys(%Array)));
	
	# Close file
	
	close(OUTPUT);
}

# Print end message

$Duration = time - $^T;

print "\nList Items finished in $Duration seconds.\n";
