#!/usr/local/bin/perl -w

# LSI.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directories (table, list)

$TableDirectory = 'Equivalents';
$ListDirectory = 'Unwanted';

# Files (table, list)

$TableName = 'Hebrews';
$ListName = 'PuncLev';

# Data field number

$Data = 1;

# Descriptive field number

$Desc = 1;

# Key words

@Descriptions = (".");

# Mode (1 = positive; -1 = negative)

$Mode = -1;

# END SPECIFICATIONS

# Print introduction

print "List Specified Items:\n\n";

print "This program lists contents of a specified data field for\n";
print "each record of a table in which key words are matched in a\n";
print "specified descriptive field of the same record.\n\n";

print "Notes:\n\n";

print "(1) This program has a positive and negative mode:\n";
print "    (+) List data field if described by key words\n";
print "    (-) List data field unless described by key words.\n";
print "(2) Key word matching is case-sensitive.\n";
print "(3) Table records must have the following format:\n";
print "    Field 1 TAB Field 2 TAB Field 3 TAB ... NEWLINE\n";
print "    The output list contains one item per line.\n\n";

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known" }

# Make paths to directories

$TablePath = $Dir.$TableDirectory.$Dir;
$ListPath = $Dir.$ListDirectory.$Dir;
unless($System eq 'MAC')
{
	$TablePath = '.'.$TablePath;
	$ListPath = '.'.$ListPath;
}

# Make list directory

mkdir($ListDirectory, 0770);

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input table: $TablePath$TableName\n";
print "Output list: $ListPath$ListName\n\n";

print "Data field number: $Data.\n";
print "Descriptive field number: $Desc.\n\n";

print "Key words:\n";
foreach $Item (@Descriptions) { print "$Item\n" }

print "\nMode:\n";
if($Mode == 1)
{ print "(+) List data field if described by key words.\n" }
elsif($Mode == -1)
{ print "(-) List data field unless described by key words.\n" }
else { die "Error: Incorrect mode specification" }

# Read table file

open(INPUT, '<'.$TablePath.$TableName) or die
"Error: Can't open table file $TablePath$TableName";

@Table = <INPUT>;

# Close table file

close(INPUT);

# Remove newlines

chomp(@Table);

# Check for enough fields

foreach $Record (@Table)
{
	@Fields = split(/\t/, $Record);
	if((@Fields < $Data) or (@Fields < $Desc))
	{ die "Error: Not enough table fields: $Record" }
}

# BUILD LIST

print "\nBuild list:\n";

@List = ();
@New = @Table;
@Table = ();
$Data -= 1;
$Desc -= 1;

foreach $Description (@Descriptions)
{
	print "$Description\n";
	$Pattern = quotemeta($Description);
	
	@Old = @New;
	@New = ();
	
	while(@Old)
	{
		# Get record
		
		$Record = shift(@Old);
		
		# Split into item, equivalent, description, ...
		
		@Fields = split(/\t/, $Record);
		
		# Add item to list and discard record if match found.
		# Otherwise, keep record for next search.
		
		if($Fields[$Desc] =~ /$Pattern/)
		{ push(@List, $Fields[$Data]) }
		else { push(@New, $Record) }
	}
}

# Open list file for writing if safe to do so

if(-e $ListPath.$ListName)
{ die "Error: List file $ListName already exists" }

open(OUTPUT, '>'.$ListPath.$ListName) or die
	"Error: Can't write to list file $ListPath$ListName";
	
print "->  $ListPath$ListName\n";

# Obtain output list

if($Mode == 1)	# Positive mode
{ @List = sort(@List) }
else			# Negative mode
{
	@List = ();
	
	foreach $Record (@New)
	{
		@Fields = split(/\t/, $Record);
		push(@List, $Fields[$Data]);
	}
	
	@List = sort(@List);
}

# Write output to file

print OUTPUT join("\n", @List), "\n";

# Close file

close(OUTPUT);

# Print end message

$Duration = time - $^T;

print "\nList Specified Items finished in $Duration seconds.\n";
