#!/usr/local/bin/perl -w

# Punctuation.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directory (input)

$InputDirectory = 'Transcribed';

# Division marker delimiters

@Division = (
	'<',	# Open delimiter
	'>'		# Close delimiter
	);

# Tag delimiters

@Tag = (
	'[',	# Start-tag open delimiter
	'[/',	# End-tag open delimiter
	']'		# Tag close delimiter
	);

# Concatenation marker

$Concatenation = '=';

# END SPECIFICATIONS

# Print introduction

print "Punctuation: List possible punctuation marks.\n\n";

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input directory: $InputDirectory.\n\n";

# Make division marker components

$DMO = $Division[0];	# Division Marker Open
$DMC = $Division[1];	# Division Marker Close

print "Division markers should look like this:\n",
	$DMO, "DIVISION MARKER", $DMC, "\n\n";

# Make tag components
	
$TSO = $Tag[0];	# Tag Start Open
$TEO = $Tag[1];	# Tag End Open
$TC = $Tag[2];	# Tag Close
	
print "Tags should look like this (gi = generic identifier):\n",
	$TSO, "gi", $TC, "TAGGED TEXT", $TEO, "gi", $TC, "\n\n";

print "Concatenation marker:\n",
	$Concatenation, "\n\n";

# Make patterns for regular expressions
	
$DMO = quotemeta($DMO);
$DMC = quotemeta($DMC);
$TSO = quotemeta($TSO);
$TEO = quotemeta($TEO);
$TC = quotemeta($TC);
$CON = quotemeta($Concatenation);

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known\n" }

# Make path to directory

$InputPath = $Dir.$InputDirectory.$Dir;
unless($System eq 'MAC') { $InputPath = '.'.$InputPath }

# Open input directory

opendir(INDIR, $InputDirectory) or die
	"Can't open input directory $InputDirectory";

# Make input directory contents list

@Contents = readdir(INDIR);

# Remove . and .. from directory list

unless($System eq 'MAC') { splice(@Contents, 0, 2) }

# LIST POSSIBLE PUNCTUATION

print "Search for possible punctuation:\n\n";

# Initialise punctuation array

@Punctuation = ();

foreach $InputName (@Contents)
{
	# Open input file for reading
	
	open(INPUT, '<'.$InputPath.$InputName) or die
		"Can't open input file $InputPath$InputName\n";
		
	print $InputPath, $InputName, "\n";
	
	# Make transcription into one string
	
	@InputList = <INPUT>;
	$Input = join('', @InputList);
	
	# Close file
	
	close(INPUT);
	
	# Make output string
	
	$Output = $Input;
	
	# Make punctuation string
	
	while($Output =~ /(\W)\s/g)
	{
		$Match = quotemeta($1);
		
		if($Match !~ /[$DMC$TC$CON\s]/)
		{
			if(not grep(/$Match/, @Punctuation))
			{
				push(@Punctuation, $1);
				printf("%s : %d\n", $1, ord($1));
			}
		}
	}
}

# List possible punctuation

@Punctuation = sort @Punctuation;

print "\nPossible punctuation marks and ASCII values:\n";

foreach $Item (@Punctuation)
{ printf("%s : %d\n", $Item, ord($Item)) }

# Print end message
	
$Duration = time - $^T;

print "\nPunctuation finished in $Duration seconds.\n";
