#!/usr/local/bin/perl -w

# Tabulate.pl

# SPECIFICATIONS

# System (Mac, UNIX, or PC)

$System = 'Mac';

# Directories (input, output)

$InputDirectory = 'Formatted';
$OutputDirectory = 'Equivalents';

# Files (standard text, output)

$Standard = 'UBS4';
$OutputName = 'Rudimentary';

# Division marker delimiters

@Division = (
	'<',	# Open delimiter
	'>'		# Close delimiter
	);

# END SPECIFICATIONS

# Print introduction

print "Tabulate:\n\n";

print "This program builds a rudimentary equivalents table:\n";
print "- list items found in witness files\n";
print "- delete items found in specified standard text\n";
print "- add adjacent list of duplicates\n";
print "- output item, duplicate, blank, and location.\n\n";

print "Notes:\n\n";

print "(1) Item refers to a division marker, a word, a newline\n";
print "    character, or a punctuation mark.\n";
print "(2) Witness files must be properly formatted.\n";
print "    (The Format utility does this.)\n";
print "(3) Location is that of the first occurrence encountered.\n";
print "(4) Output is formatted as follows:\n";
print "    Item TAB Duplicate TAB Blank TAB Location NEWLINE\n";
print "    The blank field is for descriptive notes.\n\n";

# Show specifications

print "Specifications:\n\n";

print "$System version.\n\n";

print "Input directory: $InputDirectory\n";
print "Standard text: $Standard\n";
print "Output directory: $OutputDirectory\n";
print "Table name: $OutputName\n\n";

print "Division markers should look like this:\n",
	"$Division[0]DIVISION MARKER$Division[1]\n\n";

# Make output directory

mkdir($OutputDirectory, 0770);

# Select directory character

$System = uc($System);	# Make all upper-case
if($System eq 'MAC') { $Dir = ':' }
elsif($System eq 'UNIX') { $Dir = '/' }
elsif($System eq 'PC') { $Dir = '\\' }
else { die "Error: Directory character for $System not known\n" }

# Make paths to directories

$InputPath = $Dir.$InputDirectory.$Dir;
$OutputPath = $Dir.$OutputDirectory.$Dir;
unless($System eq 'MAC')
{
	$InputPath = '.'.$InputPath;
	$OutputPath = '.'.$OutputPath;
}

# BUILD EQUIVALENTS TABLE

print "Build equivalents table:\n";

# Open input directory

opendir(INDIR, $InputDirectory) or die
	"Error: Can't open input directory $InputDirectory.\n";

# Make input directory contents list

@Contents = readdir(INDIR);

# Remove . and .. from UNIX directory list

unless($System eq 'MAC') { splice(@Contents, 0, 2) }

# Initialise

$DMO = quotemeta($Division[0]);
$DMC = quotemeta($Division[1]);
%Array = ();

# ADD ITEMS

foreach $InputName (@Contents)
{
	# Skip if standard
	
	if($InputName eq $Standard) { next }
	
	# Open input file for reading
	
	open(INPUT, '<'.$InputPath.$InputName) or die
		"Error: Can't open input file $InputPath$InputName\n";
		
	print "<- $InputPath$InputName\n";
	
	while(<INPUT>)
	{
		# Set location
		
		if($_ =~ /^($DMO.*$DMC)\n$/)
		{ $Location = $InputName." ".$1 }
		
		# Add item to array if not previously encountered
		
		elsif($_ =~ /^(.+)\t.+\n$/)
		{ unless(exists($Array{$1})) { $Array{$1} = $Location } }
		
		else { die "Error: Incorrect input format: $_.\n" }
	}
	
	# Close input file
	
	close(INPUT);
}

# DELETE STANDARD ITEMS

# Open standard file for reading

$InputName = $Standard;

open(INPUT, '<'.$InputPath.$InputName) or die
	"Error: Can't open standard file $InputName.\n";
	
print "Delete standard items found in $InputName\n";

while(<INPUT>)
{
	if(($_ =~ /($DMO.+$DMC)\n/) or ($_ =~ /(.+)\t.+\n/))
	{ delete($Array{$1}) }
	else
	{ die "Error: Incorrect input format: $_.\n" }
}

# Close file

close(INPUT);

# Open output file for writing if safe to do so

if(-e $OutputPath.$OutputName)
{ die "Error: Output file $OutputName already exists.\n" }

open(OUTPUT, '>'.$OutputPath.$OutputName) or die
	"Error: Can't write to output file $OutputPath$OutputName.\n";
print "->  $OutputPath$OutputName\n";

# Write output file

foreach $Item (sort(keys(%Array)))
{ print OUTPUT "$Item\t$Item\t\t$Array{$Item}\n" }

# Close file

close(OUTPUT);

# Print end message

$Duration = time - $^T;

print "\nTabulate finished in $Duration seconds.\n";
